@@ -27,7 +27,8 @@
 #include <sys/types.h>
 #include <libraw1394/raw1394.h>
 #include <stdio.h>
-
+#include <sys/time.h>
+#include <stdlib.h>
 
 /* Enumeration of data speeds */
 enum
@@ -200,11 +201,21 @@ enum
 /* Return values for visible functions*/
 #define DC1394_SUCCESS               1
 #define DC1394_FAILURE              -1
+#define DC1394_NO_FRAME             -2
 #define DC1394_NO_CAMERA            0xffff
 
 /* Parameter flags for dc1394_setup_format7_capture() */
 #define QUERY_FROM_CAMERA -1
 #define USE_MAX_AVAIL     -2
+#define USE_RECOMMENDED   -3
+
+/* The video1394 policy: blocking (wait for a frame forever)
+   or polling (returns if no frames in buffer */
+typedef enum
+{ 
+  VIDEO1394_WAIT=0,
+  VIDEO1394_POLL
+} dc1394videopolicy_t;
 
 /* Yet another boolean data type */
 typedef enum
@@ -234,11 +245,20 @@ typedef struct __dc1394_cam_cap_struct
     int quadlets_per_frame;
     int quadlets_per_packet;
     /* components needed for the DMA based video capture */
-    unsigned char * dma_ring_buffer;
+    const unsigned char * dma_ring_buffer;
     int dma_buffer_size;
     int dma_frame_size;
     int num_dma_buffers;
     int dma_last_buffer;
+    int num_dma_buffers_behind;
+    const char * dma_device_file;
+    int dma_fd;
+    int port;
+    struct timeval filltime;
+    int dma_extra_count;
+    unsigned char * dma_extra_buffer;
+    int do_extra_buffering;
+    int drop_frames;
 } dc1394_cameracapture ;
 
 typedef struct __dc1394_misc_info
@@ -261,6 +281,7 @@ typedef struct __dc1394_feature_info_struct
     unsigned int feature_id;
     dc1394bool_t available;
     dc1394bool_t one_push;
+    dc1394bool_t absolute_capable;
     dc1394bool_t readout_capable;
     dc1394bool_t on_off_capable;
     dc1394bool_t auto_capable;
@@ -278,6 +299,12 @@ typedef struct __dc1394_feature_info_struct
     int BU_value;
     int RV_value;
     int target_value;
+
+    dc1394bool_t abs_control;
+    float abs_value;
+    float abs_max;
+    float abs_min;
+
 } dc1394_feature_info;
 
 typedef struct __dc1394_feature_set_struct 
@@ -292,6 +319,18 @@ extern const char *dc1394_feature_desc[NUM_FEATURES];
 extern "C" {
 #endif
 
+/*****************************************************
+ Direct register manipulation functions.
+ Use with caution, this might wreak your camera.
+ *****************************************************/
+int
+SetCameraControlRegister(raw1394handle_t handle, nodeid_t node,
+                         octlet_t offset, quadlet_t value);
+
+int
+GetCameraControlRegister(raw1394handle_t handle, nodeid_t node,
+                         octlet_t offset, quadlet_t *value);
+
 /*****************************************************
  dc1394_get_camera_feature_set
 
@@ -352,6 +391,14 @@ dc1394_print_feature_set(dc1394_feature_set *features);
 raw1394handle_t 
 dc1394_create_handle(int port);
 
+/***************************************************
+ dc1394_destroy_handle
+
+ Destroy the handle, including the userdata 
+***************************************************/
+int
+dc1394_destroy_handle(raw1394handle_t handle);
+
 
 /*****************************************************
  dc1394_get_camera_nodes
@@ -369,6 +416,8 @@ nodeid_t*
 dc1394_get_camera_nodes(raw1394handle_t handle, int *numCameras,
                         int showCameras);
 
+#define dc1394_free_camera_nodes free
+
 /*****************************************************
  dc1394_get_sorted_camera_nodes
 
@@ -430,9 +479,11 @@ dc1394_query_revision(raw1394handle_t handle, nodeid_t node, int mode,
 int
 dc1394_query_basic_functionality(raw1394handle_t handle, nodeid_t node,
                                  quadlet_t *value);
+/*
 int
 dc1394_query_feature_control(raw1394handle_t handle, nodeid_t node,
                              unsigned int feature, unsigned int *availability);
+*/
 int
 dc1394_query_advanced_feature_offset(raw1394handle_t handle, nodeid_t node,
                                      quadlet_t *value);
@@ -500,6 +551,13 @@ int
 dc1394_unset_multi_shot(raw1394handle_t handle, nodeid_t node);
 
 
+int
+dc1394_get_one_shot(raw1394handle_t handle, nodeid_t node, dc1394bool_t *is_on);
+
+int
+dc1394_get_multi_shot(raw1394handle_t handle, nodeid_t node, dc1394bool_t *is_on,
+		      unsigned int *numFrames);
+
 /* Get/Set the values of the various features on the camera */
 int
 dc1394_get_brightness(raw1394handle_t handle, nodeid_t node,
@@ -695,6 +753,9 @@ dc1394_dma_setup_capture(raw1394handle_t handle, nodeid_t node,
                          int channel, int format, int mode,
                          int speed, int frame_rate, 
                          int num_dma_buffers,
+			 int do_extra_buffering,
+                         int drop_frames,
+                         const char *dma_device_file,
                          dc1394_cameracapture *camera);
 
 /*****************************************************
@@ -716,25 +777,31 @@ int
 dc1394_dma_unlisten(raw1394handle_t handle,
                           dc1394_cameracapture *camera);
 
-/*****************************************************
+/****************************************************
  dc1394_dma_single_capture
 
- This captures a frame from the given camera
+ This captures a frame from the given camera. Two
+ policies are available: wait for a frame or return
+ if no frame is available (POLL)
 *****************************************************/
 int 
 dc1394_dma_single_capture(dc1394_cameracapture *camera);
 
-/*****************************************************
+int
+dc1394_dma_single_capture_poll(dc1394_cameracapture *camera);
+
+/****************************************************
  dc1394_dma_multi_capture
 
- This capture a frame from each of the cameras passed
- in cams.  After you are finished with the frame, you
- must return the buffer to the pool by calling
- dc1394_dma_done_with_buffer.
+ This captures a frame from the given camera. Two
+ policies are available: wait for a frame or return
+ if no frame is available (POLL)
 *****************************************************/
-int
-dc1394_dma_multi_capture(dc1394_cameracapture *cams,int num);
+int 
+dc1394_dma_multi_capture(dc1394_cameracapture *camera, int num);
 
+int
+dc1394_dma_multi_capture_poll(dc1394_cameracapture *camera, int num);
 /*****************************************************
  dc1394_dma_done_with_buffer
 
@@ -886,9 +953,13 @@ dc1394_get_trigger_on_off(raw1394handle_t handle, nodeid_t node,
  *                   or USE_MAX_AVAIL
  *   \param height   area of interest height or QUERY_FROM_CAMERA
  *                   or USE_MAX_AVAIL
+ *   \param camera   (out) structure containing the returned parameters
+ *                   from the camera. Memory for this struct must be
+ *                   allocated by the calling  function
  *
  *   \return DC1394_SUCCESS or DC1394_FAILURE
- *======================================================================*/
+ */
+/*======================================================================*/
 int
 dc1394_setup_format7_capture(raw1394handle_t handle, nodeid_t node,
                              int channel, int mode, int speed,
@@ -898,6 +969,52 @@ dc1394_setup_format7_capture(raw1394handle_t handle, nodeid_t node,
                              dc1394_cameracapture * camera);
   
 
+/*======================================================================*/
+/*!
+ *   setup capture for format7 (FORMAT_SCALABLE_IMAGE_SIZE) mode using
+ *   the dma engine.  Should be much faster than the above
+ *   routines. For some parameters you may pass QUERY_FROM_CAMERA
+ *   (which means query this value from the camera and maybe adjust it
+ *   to the new conditions) or USE_MAX_AVAIL (which means query the
+ *   maximum availible value for this parameter from camera and use
+ *   this)
+ *
+ *   \param handle   handle for raw1394 port
+ *   \param node     node of the camera
+ *   \param channel  iso channel for data transmission (0 ... 15)
+ *   \param mode     mode for camera operation 
+ *                   (MODE_FORMAT7_0 ... MODE_FORMAT7_7)
+ *   \param speed    transmission speed (SPEED_100 ... SPEED_400)
+ *   \param bytes_per_packet number of bytes per packet can be used to
+ *                   control the framerate or QUERY_FROM_CAMERA or
+ *                   USE_MAX_AVAIL
+ *   \param left     area of interest start column or QUERY_FROM_CAMERA
+ *   \param top      area of interest start row or QUERY_FROM_CAMERA
+ *   \param width    area of interest width or QUERY_FROM_CAMERA
+ *                   or USE_MAX_AVAIL
+ *   \param height   area of interest height or QUERY_FROM_CAMERA
+ *                   or USE_MAX_AVAIL
+ *   \param num_dma_buffers number of buffers for the dma ring buffer
+ *   \param camera   (out) structure containing the returned parameters
+ *                   from the camera. Memory for this struct must be
+ *                   allocated by the calling  function
+ *
+ *   \return DC1394_SUCCESS or DC1394_FAILURE
+ */
+/*======================================================================*/
+int
+dc1394_dma_setup_format7_capture(raw1394handle_t handle, nodeid_t node,
+                                 int channel, int mode, int speed,
+                                 int bytes_per_packet,
+                                 unsigned int left, unsigned int top,
+                                 unsigned int width, unsigned int height,
+                                 int num_dma_buffers,
+				 int do_extra_buffering,
+				 int drop_frames,
+				 const char *dma_device_file,
+                                 dc1394_cameracapture *camera);
+  
+
 int
 dc1394_query_format7_max_image_size(raw1394handle_t handle, nodeid_t node,
 				    unsigned int mode,
@@ -923,7 +1040,7 @@ dc1394_query_format7_image_size(raw1394handle_t handle, nodeid_t node,
 
 int
 dc1394_query_format7_color_coding_id(raw1394handle_t handle, nodeid_t node,
-				     unsigned int mode, quadlet_t *value);
+				     unsigned int mode, unsigned int *color_id);
 
 int
 dc1394_query_format7_color_coding(raw1394handle_t handle, nodeid_t node,
@@ -935,7 +1052,7 @@ dc1394_query_format7_pixel_number(raw1394handle_t handle, nodeid_t node,
 
 int
 dc1394_query_format7_total_bytes(raw1394handle_t handle, nodeid_t node,
-				 unsigned int mode, unsigned int *total_bytes);
+				 unsigned int mode, unsigned long long int *total_bytes);
 
 int
 dc1394_query_format7_packet_para(raw1394handle_t handle, nodeid_t node,
@@ -965,6 +1082,76 @@ int
 dc1394_set_format7_byte_per_packet(raw1394handle_t handle, nodeid_t node,
 				   unsigned int mode,
                                    unsigned int packet_bytes);
+
+int
+dc1394_query_format7_value_setting(raw1394handle_t handle, nodeid_t node,
+				   unsigned int mode,
+				   unsigned int *present,
+				   unsigned int *setting1,
+				   unsigned int *err_flag1,
+				   unsigned int *err_flag2);
+int
+dc1394_set_format7_value_setting(raw1394handle_t handle, nodeid_t node,
+				 unsigned int mode);
+
+int
+dc1394_query_format7_recommended_byte_per_packet(raw1394handle_t handle, nodeid_t node,
+						  unsigned int mode,
+						  unsigned int *bpp);
+
+int
+dc1394_query_format7_packet_per_frame(raw1394handle_t handle, nodeid_t node,
+				      unsigned int mode,
+				      unsigned int *ppf);
+
+int
+dc1394_query_format7_packet_per_frame(raw1394handle_t handle, nodeid_t node,
+				      unsigned int mode,
+				      unsigned int *ppf);
+
+int
+dc1394_query_format7_unit_position(raw1394handle_t handle, nodeid_t node,
+				   unsigned int mode,
+				   unsigned int *horizontal_pos,
+				   unsigned int *vertical_pos);
+
+/**********************************
+ *   ABSOLUTE SETTING FUNCTIONS   *		     
+ **********************************/
+
+
+int
+dc1394_query_absolute_feature_min_max(raw1394handle_t handle, nodeid_t node,
+				      unsigned int feature,
+				      float *min, float *max);
+
+int
+dc1394_query_absolute_feature_value(raw1394handle_t handle, nodeid_t node,
+				    int feature, float *value);
+
+int
+dc1394_set_absolute_feature_value(raw1394handle_t handle, nodeid_t node,
+				  int feature, float value);
+
+int
+dc1394_query_absolute_control(raw1394handle_t handle, nodeid_t node,
+			    unsigned int feature, dc1394bool_t *value);
+
+int
+dc1394_absolute_setting_on_off(raw1394handle_t handle, nodeid_t node,
+			       unsigned int feature, unsigned int value);
+
+int
+dc1394_has_absolute_control(raw1394handle_t handle, nodeid_t node,
+			    unsigned int feature, dc1394bool_t *value);
+
+
+int
+dc1394_get_bandwidth_usage(raw1394handle_t handle, nodeid_t node, unsigned int *bandwidth);
+
+int
+dc1394_get_camera_port(raw1394handle_t handle);
+
 #ifdef __cplusplus
 }
 #endif
